@@ -7,6 +7,7 @@
 #include <stdio.h>
 #include <unistd.h>
 #include <string.h>
+#include <ctype.h>
 #include <fcntl.h>
 #include <errno.h>
 #include <a.out.h>
@@ -152,13 +153,29 @@ void do_map_drive(void)
 	}
 	if (curr_drv_map == DRVMAP_SIZE)
 	    cfg_error("Too many drive mappings (more than %d)",DRVMAP_SIZE);
-	if (verbose > 3)
+	if (verbose > 1)
 	    printf("  Mapping BIOS drive 0x%02x to 0x%02x\n",from,to);
 	drv_map[curr_drv_map++] = (to << 8) | from;
     }
     cfg_unset(cf_other,"map-drive");
 }
 
+/*
+   Return the derived "table=", if possible
+*/
+static char npart[10];
+char *boot_table (char *boot)
+{
+    char *part = NULL;
+    if (strlen(boot)==9 && 
+	(strncmp(boot,"/dev/hd",7)==0 || strncmp(boot,"/dev/sd",7)==0) &&
+	    boot[8]>='1' && boot[8]<='4') {
+		part = strcpy(npart, boot);
+		part[8] = 0;
+    }
+    return part;
+}
+
 
 #define PART(s,n) (((struct partition *) (s)[0].par_c.ptable)[(n)])
 
@@ -170,10 +187,24 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
     BOOT_SECTOR buff[SETUPSECS-1];
     struct stat st;
     char *pos;
-    char npart[10];
     int i;
+    int letter = 0;
 
     if (!loader) loader = DFL_CHAIN;
+    if (part && strlen(part)>0 && strlen(part)<=2) {
+    	if (part[1]==0 || part[1]==':') {
+    	    letter = toupper(part[0]);
+    	    if (letter>='C' && letter<='Z') {
+    	    	letter += 0x80-'C';
+    	    	part = NULL;
+    	    }
+    	    else letter = 0;
+    	}
+    }
+#if 1
+    if (!part) part = boot_table(boot);
+    /* part may still be NULL */
+#else
     if (!part) {   /*  "/dev/hdXN and /dev/sdXN have default partition tables -- JRC */
         if (strlen(boot)==9 && 
        	  (strncmp(boot,"/dev/hd",7)==0 || strncmp(boot,"/dev/sd",7)==0) &&
@@ -182,9 +213,14 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	    part[8] = 0;
 	}
     }
+#endif
     if (verbose > 0)
 	printf("Boot other: %s%s%s, loader %s\n",boot,part ? ", on " : "",part
 	  ? part : "",loader);
+
+    if (!cfg_get_flag(cf_other, "change"))
+        do_cr_auto();
+
     if (cfg_get_flag(cf_other,"unsafe")) {
 	(void) geo_open_boot(&geo,boot);
 	if (part) die("TABLE and UNSAFE are mutually incompatible.");
@@ -194,11 +230,11 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
 	if (fstat(b_fd,&st) < 0)
 	    die("fstat %s: %s",boot,strerror(errno));
 	if (!geo.file) part_verify(st.st_rdev,0);
-	if (lseek(b_fd,(long) BOOT_SIG_OFFSET,0) < 0)
+	if (lseek(b_fd,(long) BOOT_SIG_OFFSET,SEEK_SET) < 0)
 	    die("lseek %s: %s",boot,strerror(errno));
-	if ((size = read(b_fd,(char *) &magic,2)) != 2)
+	if ((size = read(b_fd, (char *)&magic, 2)) != 2) {
 	    if (size < 0) die("read %s: %s",boot,strerror(errno));
-	    else die("Can't get magic number of %s",boot);
+	    else die("Can't get magic number of %s",boot); }
 	if (magic != BOOT_SIGNATURE)
 	    die("First sector of %s doesn't have a valid boot signature",boot);
     }
@@ -214,6 +250,7 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
         p_fd = -1; /* pacify GCC */
         PART(buff,0).boot_ind = geo.device;
         PART(buff,0).start_sect = geo.start;     /* pseudo partition table */
+        if (verbose > 0) printf("Pseudo partition start: %d\n", geo.start);
     }
     else {
 	if ((p_fd = open(part,O_RDONLY)) < 0)
@@ -246,6 +283,10 @@ void boot_other(char *loader,char *boot,char *part,IMAGE_DESCR *descr)
     }
     (void) close(l_fd);
     buff[0].par_c.drive = geo.device;
+    buff[0].par_c.head = letter ? letter : geo.device;
+     		/* IBM boot manager passes drive letter in offset 0x25 */
+    if (verbose>=5) printf("boot_other:  drive=0x%02x   logical=0x%02x\n",
+    			buff[0].par_c.drive, buff[0].par_c.head);
     drv_map[curr_drv_map] = 0;
     prt_map[curr_prt_map] = 0;
     pos = (char *) buff+buff[0].par_c.drvmap;
