@@ -44,6 +44,7 @@ static char temp_map[PATH_MAX+1];
 static char *fallback[MAX_IMAGES];
 static int fallbacks = 0;
 
+static void bsect_do_scheme(char *scheme, unsigned char *table);
 
 static void open_bsect(char *boot_dev)
 {
@@ -93,8 +94,9 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
       "9600\000";
     GEOMETRY geo;
     struct stat st;
+    int i;
     int i_fd,m_fd,kt_fd,sectors;
-    char *message,*colon,*serial,*walk,*this,*keytable;
+    char *message,*colon,*serial,*walk,*this,*keytable,*scheme;
     unsigned char table[SECTOR_SIZE];
     unsigned long timestamp;
 
@@ -117,7 +119,8 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
 	map_begin_section(); /* no access to the (not yet open) map file
 		required, because this map is built in memory */
 	map_add(&geo,1,(st.st_size+SECTOR_SIZE-1)/SECTOR_SIZE-1);
-	sectors = map_write(bsect.par_1.secondary,MAX_SECONDARY,1);
+	sectors = map_write(bsect.par_1.secondary,
+				nelem(bsect.par_1.secondary), 1);
 	if (verbose > 1)
 	    printf("Secondary loader: %d sector%s.\n",sectors,sectors == 1 ?
 	      "" : "s");
@@ -195,8 +198,6 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
     else if (timeout*100/55 >= 0xffff) die("Maximum timeout is one hour.");
 	else bsect.par_1.timeout = timeout*100/55;
     if (!(keytable = cfg_get_strg(cf_options,"keytable"))) {
-	int i;
-
 	for (i = 0; i < 256; i++) table[i] = i;
     }
     else {
@@ -206,6 +207,14 @@ void bsect_open(char *boot_dev,char *map_file,char *install,int delay,
 	    die("%s: bad keyboard translation table",keytable);
 	(void) close(kt_fd);
     }
+    for (i=256; i<SECTOR_SIZE; i++) table[i] = 0;
+    if ((scheme = cfg_get_strg(cf_options,"menu-scheme"))) {
+    	bsect_do_scheme(scheme, table+256);
+    }
+    if ((scheme = cfg_get_strg(cf_options,"menu-title"))) {
+    	strncpy(table+256+9, scheme, 37);
+    	table[256+8] = strlen(table+256+9);
+    }
     map_begin_section();
     map_add_sector(table);
     (void) map_write(&bsect.par_1.keytab,1,0);
@@ -293,13 +302,14 @@ static void bsect_common(IMAGE_DESCR *descr)
     if (cfg_get_flag(cf_kernel,"read-write") || cfg_get_flag(cf_options,
       "read-write")) strcat(options,"rw ");
     if ((root = cfg_get_strg(cf_kernel,"root")) || (root = cfg_get_strg(
-      cf_options,"root")))
+      cf_options,"root")))  {
 	if (strcasecmp(root,"current")) 
 	    sprintf(strchr(options,0),"root=%x ",dev_number(root));
 	else {
 	    if (stat("/",&st) < 0) pdie("stat /");
 	    sprintf(strchr(options,0),"root=%x ",(unsigned int) st.st_dev);
 	}
+      }	
     if ((ram_disk = cfg_get_strg(cf_kernel,"ramdisk")) || (ram_disk =
       cfg_get_strg(cf_options,"ramdisk")))
 	sprintf(strchr(options,0),"ramdisk=%d ",to_number(ram_disk));
@@ -352,6 +362,7 @@ static void bsect_common(IMAGE_DESCR *descr)
 #else
 	if (descr->flags & FLAG_LOCK)
 	    die("LOCK and FALLBACK are mutually exclusive");
+	else descr->flags |= FLAG_FALLBACK;
 	*(unsigned short *) fallback_buf = DC_MAGIC;
 	strcpy(fallback_buf+2,fback);
 	fallback[fallbacks++] = stralloc(fback);
@@ -589,3 +600,48 @@ void bsect_uninstall(char *boot_dev,char *backup_file,int validate)
 	}
     exit(0);
 }
+
+
+static void bsect_do_scheme(char *scheme, unsigned char *table)
+{
+    static char khar[] = "kbgcrmywKBGCRMYW";
+    unsigned int fg, bg;
+    int i;
+    /* order of color attributes is:
+          text, hilighted text, border, title
+    */
+#define color(c) ((int)strchr(khar,(int)(c))-(int)khar)
+    for (i=0; i<8; i++) table[i] = 0;
+    for (i=0; i<4 && *scheme; i++) {
+    	fg = color(*scheme);
+    	if (fg>16) die("Invalid menu-scheme color: '%c'", *scheme);
+    	if (*++scheme) bg = color(*scheme);
+    	else {
+	    pdie("Invalid menu-scheme syntax");
+    	}
+    	if (bg>16) die("Invalid menu-scheme color: '%c'", *scheme);
+    	if (*++scheme) {
+    	    if (ispunct(*scheme)) scheme++;
+    	    else pdie("Invalid menu-scheme punctuation");
+    	}
+    	if (bg>8 && !nowarn)
+    	    fprintf(stderr, "Warning: menu-scheme BG color may not be intensified\n");
+    	table[i+4] = ((bg<<4) | fg) & 0x7F;
+    }
+    /* check on the TEXT color */
+    if (table[4] == 0) {
+        if (!nowarn)
+            fprintf(stderr, "Warning: menu-scheme \"black on black\" changed to "
+        	"\"white on black\"\n");
+        table[4] = 0x07;
+    }
+    /* check on the HIGHLIGHT color */
+    if (table[5] == 0)  table[5] = ((table[4]<<4)&0x70) | ((table[4]>>4)&0x0f);
+    /* check on the BORDER color */
+    if (table[6] == 0)  table[6] = table[4];
+    /* check on the TITLE color */
+    if (table[7] == 0)  table[7] = table[6];
+    
+    strncpy(table, "MENU", 4);
+#undef color
+}
