@@ -32,6 +32,7 @@ extern void do_change(void);
 extern void do_cr_type(void);
 extern void do_cr_reset(void);
 extern void do_cr_part(void);
+extern void do_cr_auto(void);
 
 
 CONFIG cf_top[] = {
@@ -80,6 +81,8 @@ CONFIG cf_options[] = {
   { cft_flag, "change-rules",	do_cr,		NULL,NULL },
   { cft_flag, "single-key",	NULL,		NULL,NULL },
   { cft_strg, "keytable",	NULL,		NULL,NULL },
+  { cft_strg, "menu-scheme",	NULL,		NULL,NULL },
+  { cft_strg, "menu-title",	NULL,		NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
 CONFIG cf_all[] = {
@@ -148,6 +151,7 @@ CONFIG cf_change_rule[] = {
 
 CONFIG cf_change[] = {
   { cft_strg, "partition",	do_cr_part,	NULL,NULL },
+  { cft_flag, "automatic",	do_cr_auto,	NULL,NULL },
   { cft_end,  NULL,		NULL,		NULL,NULL }};
 
 CONFIG cf_change_dsc[] = {
@@ -394,7 +398,7 @@ static int cfg_do_set(CONFIG *table,char *item,char *value,int copy,
 		cfg_error("'%s' doesn't have a value",walk->name);
 	    if (!value && walk->type == cft_strg)
 		cfg_error("Value expected for '%s'",walk->name);
-	    if (walk->data) 
+	    if (walk->data) {
 		if (walk->context == context)
 		    cfg_error("Duplicate entry '%s'",walk->name);
 		else {
@@ -402,10 +406,10 @@ static int cfg_do_set(CONFIG *table,char *item,char *value,int copy,
 		    if (!copy) free(value);
 		    return 1;
 		}
+	    }
 	    if (walk->type == cft_flag) walk->data = &flag_set;
 	    else if (walk->type == cft_strg)
-		    if (copy) walk->data = stralloc(value);
-		    else walk->data = value;
+		    walk->data = copy ? stralloc(value) : value; 
 	    walk->context = context;
 	    if (walk->action) ((void (*)(void)) walk->action)();
 	    break;
