@@ -1,21 +1,23 @@
 /* geometry.c  -  Device and file geometry computation */
 
 /* Copyright 1992-1998 Werner Almesberger. See file COPYING for details. */
-
+/* Patched for linux-2.4.0 - Glibc-2.2 by Sergey Ostrovsky 11/16/2000 */
 
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/statfs.h>
 #include <stdio.h>
-#include <string.h>
-#include <errno.h>
 #include <fcntl.h>
-#include <dirent.h>
-#include <limits.h>
-#include <sys/stat.h>
-#include <sys/ioctl.h>
+#include <errno.h>
+
+#ifdef	_SYS_STATFS_H
+#define	_I386_STATFS_H	/* two versions of statfs is not good ... */
+#endif
 
 #include <linux/fs.h>
-#include <linux/fd.h>
 #include <linux/hdreg.h>
+#include <linux/fd.h>
+#include <string.h>
 
 #include "config.h"
 #include "lilo.h"
@@ -25,7 +27,6 @@
 #include "cfg.h"
 #include "md-int.h"
 
-
 #ifdef LCF_REISERFS
 #ifndef REISERFS_SUPER_MAGIC
 #define REISERFS_SUPER_MAGIC 0x52654973
@@ -614,8 +615,10 @@ int geo_comp_addr(GEOMETRY *geo,int offset,SECTOR_ADDR *addr)
 	    die("sector %d too large for linear mode"
 					" (try 'lba32' instead)", sector);
 	if (verbose > 4)
-	    printf("fd %d: offset %d -> dev 0x%02x, linear %d\n",
-		         geo->fd, offset, addr->device, sector);
+	    printf("fd %d: offset %d -> dev 0x%02x, %s %d\n",
+		         geo->fd, offset, addr->device,
+		         lba32 ? "LBA" : "linear",
+		         sector);
     }
     else {
 	addr->device = geo->device;
