@@ -314,8 +314,11 @@ fprintf(stderr,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		break;
 	    case 'I':
 		identify = param;
-		if (argc--) {
+		if (argc) {
 		    ident_opt = *argv++;
+		    argc--;
+		} else {
+		    ident_opt = "i";
 		}
 		break;
 	    case 'l':
@@ -340,9 +343,7 @@ fprintf(stderr,"argc=%d, *argv=%s, ch=%c param=%s\n", argc, *argv, ch, param);
 		query = 1;
 		break;
 	    case 'r':
-		if (!argc) usage(name);
 		new_root = param;
-		argc--;
 		break;
 	    case 'R':
 	        if (*(param = (*argv)+2)) argc++;
@@ -365,9 +366,7 @@ fprintf(stderr,"REBOOT=\"%s\"\n", reboot_arg);
 #endif
 		break;
 	    case 's':
-		if (!argc) usage(name);
 		cfg_set(cf_options,"backup",param,NULL);
-		argc--;
 		break;
 	    case 'S':
 		cfg_set(cf_options,"force-backup",param,NULL);
