@@ -21,15 +21,14 @@
 #else
 #include <linux/config.h>
 #endif
-#if 0
-#define VERSION		21 /* Boot sector, map file, and chain loader format
-			      revision. This number does not necessarily have
-			      to correspond to the version number of the entire
-			      package, but it usually does. */
-#else
+
+#define nelem(a) (sizeof(a)/sizeof((a)[0]))
+#define S2(x) #x
+#define S(x) S2(x)
+
 #include "version.h"
 #define VERSION 256*VERSION_MINOR+VERSION_MAJOR
-#endif
+
 #define MAJMIN_RAM	0x101 /* RAM disk */
 #define MAJOR_FD	2 /* floppy disks */
 #define MAJOR_HD	3 /* IDE-type hard disks */
@@ -48,8 +47,9 @@
 			  /* maximum number of images */
 #define MAX_IMAGE_NAME	15 /* maximum name length (w/o terminating NUL) */
 #define MAX_PW		15 /* maximum password length */
-#define MAX_SECONDARY	10 /* maximum number of sectors in the 2nd stage
-			     loader */
+
+#define MAX_SECONDARY 12
+
 #define SECTOR_SIZE	512 /* disk sector size */
 #ifndef BLOCK_SIZE /* may also come from linux/fs.h */
 #define BLOCK_SIZE	1024 /* disk block size */
@@ -67,9 +67,6 @@
 #define PART_MASK  15 /* largest common partition bits mask */
 #define PART_MAX    4 /* biggest primary partition number */
 
-#define PART_LINUX_MINIX  0x81	/* Linux/MINIX partition */
-#define PART_LINUX_NATIVE 0x83	/* Linux native (file system) */
-
 #define MAX_BOOT_SIZE	0x1b6	/* (leave some space for NT's and DR DOS' dirty
 				   hacks) scream if the boot sector gets any
 				   bigger */
@@ -79,10 +76,23 @@
 #define BOOT_SIGNATURE	0xaa55	/* boot signature */
 #define BOOT_SIG_OFFSET	510	/* boot signature offset */
 
-#define PART_DOS12	1	/* DOS 12 bit FAT partition type */
-#define PART_DOS16_SMALL 4	/* DOS 16 bit FAT partition type, < 32 MB */
-#define PART_DOS16_BIG	6	/* DOS 16 bit FAT partition type, >= 32 MB */
-#define PART_INVALID	98	/* invalid partition type */
+#define PART_LINUX_MINIX  0x81	/* Linux/MINIX partition */
+#define PART_LINUX_SWAP	  0x82	/* Linux swap partition */
+#define PART_LINUX_NATIVE 0x83	/* Linux native (file system) */
+#define PART_DOS_EXTD 	  5	/* DOS Extended partition */
+#define PART_WIN_EXTD_LBA 0xF	/* Win95/98 Extended partition */
+#define PART_LINUX_EXTD   0x85	/* Linux Extended partition */
+#define PART_OS2_BOOTMGR  0xA	/* OS/2 Boot Manager */
+
+#define PART_DOS12	  1	/* DOS 12 bit FAT partition type */
+#define PART_DOS16_SMALL  4	/* DOS 16 bit FAT partition type, < 32 MB */
+#define PART_DOS16_BIG	  6	/* DOS 16 bit FAT partition type, >= 32 MB */
+#define PART_HPFS	  7	/* OS/2 High Performance File System */
+#define PART_NTFS	  7	/* WinNT File System */
+#define PART_FAT32	  0xB	/* Win95/98 FAT32 partition on small disk */
+#define PART_FAT32_LBA	  0xC	/* Win95/98 FAT32 partition on large disk */
+#define PART_FAT16_LBA	  0xE	/* Win95/98 FAT16 partition on large disk */
+#define PART_INVALID	  98	/* invalid partition type */
 
 #define HIDDEN_OFF	0x10	/* type offset to hide partition (OS/2 BM) */
 #define PART_HDOS12	PART_DOS12+HIDDEN_OFF
@@ -96,7 +106,12 @@
 #define SETUPSECS	4	/* nr of setup sectors */
 #define VSS_NUM		497	/* address where variable setup size is
 				   stored */
+#ifndef LCF_LARGE_EBDA
 #define MAX_KERNEL_SECS	1024	/* absolute maximum kernel size */
+#else
+#define MAX_KERNEL_SECS	(1024-MAX_SECONDARY-5-1)
+			/* abs. max. kernel size (less 9k is gone) */
+#endif
 #define SPECIAL_SECTORS	2	/* special sectors (don't compact) at beginning
 				   of map sections */
 
@@ -109,9 +124,10 @@
 #define EX_DL_MAG	0xfe	/* magic number in DL */
 #define EX_MAG_L	0x494c	/* magic number at ES:SI, "LI" */
 #define EX_MAG_H	0x4f4c	/* magic number at ES:SI+2, "LO" */
+#define EX_MAG_HL	0x4f4c494c  /* "LILO" */
 
 #define BIOS_MAX_DEVS	2	/* BIOS devices (2 fd, 2 hd) */
-#define BIOS_MAX_HEADS  256	/* 8 bits head number */
+#define BIOS_MAX_HEADS  255	/* 8 bits head number;  NOT 256 */
 #define BIOS_MAX_CYLS   1024	/* 10 bits cylinder number */
 #define BIOS_MAX_SECS   63	/* 6 bits sector number (no, it's not 64) */
 
@@ -121,6 +137,7 @@
 #define FLAG_MODKRN	8	/* modern kernel with high memory support */
 #define FLAG_KERNEL	16	/* image is a kernel */
 #define FLAG_SINGLE	32	/* single key activation */
+#define FLAG_FALLBACK	64	/* fallback command line exists */
 
 #define VGA_NOCOVR	0x8000	/* VGA setting not overridden on command line */
 
@@ -133,6 +150,7 @@
 
 #define NEW_HDR_SIG	"HdrS"	/* setup header signature */
 #define NEW_HDR_VERSION	0x200	/* header version number */
+#define NEW2_HDR_VERSION 0x202	/* new cmdline protocol */
 #define LOADER_VERSION	1	/* loader version, for SETUP_HDR.loader */
 #define LFLAG_HIGH	1	/* SETUP_HDR.flags */
 #define LFLAG_USE_HEAP	0x80
@@ -140,70 +158,117 @@
 #define PRTMAP_SIZE	32	/* number of partition type mappings */
 #define DRVMAP_SIZE	8	/* number of drive mappings */
 
-#ifdef LILO_ASM
-
+#ifndef LILO_ASM
+#define BOOTSEG 0x07c0			/* for probe.c  */
+#else
 BOOTSEG   = 0x07C0			! original address of boot-sector
 PARTS_LOAD= 0x0600			! partition sector load address
 PARTS_SCR = 0x0800			! ditto, for non-boot partitions
 PART_TABLE= 0x07BE			! partition table
 
-FIRSTSEG  = 0x9A00
-STACKSEG  = 0x9000
-#ifndef LCF_LARGE_EBDA
-SECONDSEG = 0x9B00			! second stage loader segment
-#else
-SECONDSEG = 0x0B00			! second stage loader segment
-#endif
-
 INITSEG   = DEF_INITSEG			! we move boot here - out of the way
 SETUPSEG  = DEF_SETUPSEG		! setup starts here
 SYSSEG    = DEF_SYSSEG			! system loaded at 0x10000 (65536).
-SSDIFF	  = 0xA000
-STACK	  = 0xB000			! top of stack
-DESCR	  = 0x2200			! descriptor load area offset
-DFLCMD	  = 0x2600			! default command line
-MAP	  = 0x2000			! map load area offset
-KEYTABLE  = 0x2800			! keyboard translation table offset
+
+MAPcode   = MAX_SECONDARY*0x200		! size of secondary code
+MAP	  = MAPcode+0x200		! plus space for cmdline input
+DESCR	  = MAP+0x200	     ! 0x2200	! descriptor load area offset
+DFLCMD	  = DESCR+0x400	     ! 0x2600	! default command line
+KEYTABLE  = DFLCMD+0x200     ! 0x2800	! keyboard translation table offset
+DATAEND	  = KEYTABLE+0x200   ! 0x2A00	! PARAMLINE or Load-Low area
+
+STACKSEG  = 0x9000	     ! MUST == INITSEG for kernel 2.0.36 (and others?)
+SETUP_STACKSIZE = 2048		! stacksize for kernel setup.S
+
+#endif
+#define FIRSTSEG BOOTSEG
+
+#ifdef LILO_ASM
 
 #ifndef LCF_LARGE_EBDA
-PARMLINE  = 0x2a00			! parameter line construction area off.
+STACK	  = 2048		! amount of stack space to reserve
+PARMLINE  = DATAEND		! 0x2A00	! parameter line construction area off.
+
 #else
-PARMLINE  = 0XB000			! parameter line construction area off.
+STKseg	  = FIRSTSEG-STACKSEG
+STACK	  = STKseg<<4	! 0xB000	! top of stack
+
+SECseg    = 0x10000-DATAEND		! min place for second segment
+SECONDSEG = SECseg>>4			! seg. register contents
+!SECONDSEG = 0x0D60	! 0x0B00	! second stage loader segment
+PARMLINE  = STACK+0x200	! 0x8000	! parameter line construction area
+					! direct offset into INITSEG
+#endif
+
+#if 0
+SSDIFFseg = FIRSTSEG-STACKSEG
+SSDIFF	  = SSDIFFseg<<4     ! 0xA000	! used to address FIRSTSEG from SS
+#else
+SSDIFF	= 0
+#endif
+
+!SLA_SIZE  = 0x9E00		! setup load area size
+				! as the max. heap pointer in newer kernels
+BOOTSECT  = 0x200		! kernel bootsect.S
+
+#ifndef LCF_LARGE_EBDA
+#define SETUP_STACK_DYN  PARMLINE
+#define SLA_SIZE_DYN  SETUP_STACK_DYN-SETUP_STACKSIZE-BOOTSECT
+#else
+#define SETUP_STACK  PARMLINE
+#define SLA_SIZE  SETUP_STACK-SETUP_STACKSIZE-BOOTSECT
 #endif
 
-SECOND	  = 0x1000			! second stage loader load address
-SECOND_SS = 0xB000			! second as seen from SS:0000
-SLA_SIZE  = 0x9E00			! setup load area size
 KBBEG     = 0x41A			! beginning of keyboard buffer
 KBEND	  = 0x41C			! end of keyboard buffer
 KBLOW	  = 0x1e
 KBHIGH	  = 0x3e
 
+!
+!  Memory layout without LARGE_EBDA option
+!
 ! 0x007BE-0x007FD    64 B    partition table
-! 0x07C00-0x07DFF   0.5 kB   boot load address
-! 0x10000-0x8FFFF   512 kB   kernel
-! 0x90000-0x901FF   0.5 kB   Linux floppy boot sector
-! 0x90200-0x99FFF  39.5 kB   setup
-! 0x9A000-0x9A1FF   0.5 kB   boot sector (first stage loader)
-! 0x9A200-0x9AFFF   3.5 kB   stack
-! 0x9B000-0x9CFFF     8 kB   second stage loader
-! 0x9D000-0x9D1FF   0.5 kB   file map load area
-! 0x9D200-0x9D5FF     1 kB   descriptor table load area
-! 0x9D600-0x9D7FF   0.5 kB   default command line load area
-! 0x9D800-0x9D9FF   0.5 kB   keyboard translation table load area
-! 0x9DA00-0x9DDFF     1 kB   parameter line construction area
-! 0x9DE00-0x9F9FF   7.5 kB   scratch space
-! 0x9FC00-0x9FFFF     1 kB   driver swapper
+! 0x07C00-0x07DFF   0.5 kB   HD/FD boot load address
+! 0x10000-0x8FFFF 512.0 kB   kernel (zImage)
+! 0x90000-0x901FF   0.5 kB   kernel floppy boot sector (bootsect.S)
+! 0x90200-0x967FF  25.5 kB   kernel setup code (setup.S) and heap
+! 0x96800-0x969FF   0.5 kB   LILO stack
+! 0x96A00-0x96BFF   0.5 kB   LILO first stage loader
+! 0x96C00-0x985FF   6.5 kB   LILO second stage loader
+! 0x98600-0x987FF   0.5 kB   file map load area
+! 0x98800-0x98BFF     1 kB   descriptor table load area
+! 0x98C00-0x98DFF   0.5 kB   default command line load area
+! 0x98E00-0x98FFF   0.5 kB   keyboard translation table load area
+! 0x99000-0x991FF   0.5 kB   parameter line construction area
+! 0x99200-0x9FFFF  27.5 kB   Extended BIOS Data Area
+
+! when LILO has loaded the kernel, and control is transfered to
+! the kernel setup.S code at 0x9020:0000
+!
+! 0x007BE-0x007FD    64 B    partition table
+! 0x07C00-0x07DFF   0.5 kB   HD/FD boot load address
+! 0x10000-0x8FFFF 512.0 kB   kernel (zImage)
+! 0x90000-0x901FF   0.5 kB   kernel floppy boot sector (bootsect.S)
+! 0x90200-0x967FF  25.5 kB   kernel setup code (setup.S) and heap
+! 0x96800-0x987FF   8.0 kB   additional heap space
+! 0x98800-0x98FFF   2.0 kB   stack created for (setup.S)
+! 0x99000-0x991FF   0.5 kB   parameter line for kernel
+! 0x99200-0x9FFFF  27.5 kB   Extended BIOS Data Area
 
 CL_MAGIC_ADDR	= 0x20			! command line magic number
 CL_MAGIC	= 0xa33f		! very unusual command sequence
 CL_OFFSET	= 0x22			! command line offset
-CL_LENGTH	= 0x4e			! maximum length
-CL_KERNEL	= 256			! maximum passed to kernel
+CL_LENGTH	= 256			! maximum length
 
 ! 0x90020-0x90021     2 by   command line magic number
 ! 0x90022-0x90023     2 by   command line offset
 
-#endif
+CL_HEADER_ID	= 0x202			! "HdrS"
+CL_HDRS_VERSION	= 0x206			! 0x0201=old;  0x0202=new
+NEW_VERSION	= NEW2_HDR_VERSION	! 0x0202 for new cmdline protocol
+CL_POINTER	= 0x228			! new pointer is dword address
+
 
 #endif
+
+#endif
\ No newline at end of file
