@@ -260,8 +260,9 @@ int map_end_section(SECTOR_ADDR *addr,int dont_compact)
 	      (walk->addr.track << 8) | walk->addr.sector,(walk->addr.head << 8)
 	      | walk->addr.device);
 	    if (linear||lba32)
-		printf(", linear=%d",(walk->addr.head << 16) | (walk->addr.track
-		  << 8) | walk->addr.sector | hinib<<24);
+		printf(", %s=%d",
+		  lba32 ? "LBA" : "linear",
+		  (walk->addr.head << 16) | (walk->addr.track << 8) | walk->addr.sector | hinib<<24);
 	    printf("\n");
 	}
 	if (first) {
