@@ -6,14 +6,23 @@
 #ifndef PARTITION_H
 #define PARTITION_H
 
-void part_verify(int dev_nr,int type);
+typedef struct _change_rule {
+    const char *type;
+    unsigned char normal;
+    unsigned char hidden;
+    struct _change_rule *next;
+} CHANGE_RULE;
 
+
+void part_verify(int dev_nr,int type);
 /* Verify the partition table of the disk of which dev_nr is a partition. May
    also try to "fix" a partition table. Fail on non-Linux partitions if the
    TYPE flag is non-zero (unless IGNORE-TABLE is set too). */
 
-void preload_types(void);
+void do_cr_auto(void);
+/* do automatic change-rules */
 
+void preload_types(void);
 /* Preload some partition types for convenience */
 
 #endif
